<?php

declare(strict_types=1);

namespace Drupal\file_defaults_by_type_ui\Controller;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\file_defaults_by_type\DefaultFilesFieldManagerInterface;

/**
 * Controller for file_defaults_by_type admin section.
 */
class AdminController extends ControllerBase {

  use AutowireTrait;

  /**
   * Constructs a new AdminController object.
   *
   * @param \Drupal\file_defaults_by_type\DefaultFilesFieldManagerInterface $defaultFilesFieldManager
   *   The default files field manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    protected DefaultFilesFieldManagerInterface $defaultFilesFieldManager,
    EntityTypeManagerInterface $entityTypeManager,
  ) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Renders a list of fields default file types are are enabled.
   *
   * @return array
   *   The render array.
   */
  public function fieldList(): array {
    $fields = $this->defaultFilesFieldManager->getEnabledFields();

    $rows = [];
    $cacheable_metadata = new CacheableMetadata();

    foreach ($fields as $field) {
      $status = $this->defaultFilesFieldManager->getFieldConfigurationStatus($field);

      $row = [
        'label' => $field->getLabel(),
        'field_name' => $field->getName(),
        'entity_type' => $field->getTargetEntityTypeId(),
        'bundle' => $field->getTargetBundle(),
        'status' => $this->getStatusLabel($status),
        'operations' => [
          'data' => [
            '#type' => 'operations',
            '#links' => [],
          ],
        ],
      ];

      // The edit route has by default the same permission as this admin route,
      // but we need to take into account possible overrides.
      $edit_url = Url::fromRoute('file_defaults_by_type_ui.admin.field_default_files', [
        'field' => $field->id(),
      ]);
      $access = $edit_url->access(NULL, TRUE);
      $cacheable_metadata->addCacheableDependency($access);

      if ($access->isAllowed()) {
        $row['operations']['data']['#links']['edit'] = [
          'title' => $this->t('Edit'),
          'url' => $edit_url,
        ];
      }

      $rows[] = $row;
    }

    $build = [
      '#type' => 'table',
      '#header' => [
        'label' => $this->t('Label'),
        'machine_name' => $this->t('Field name'),
        'entity_type' => $this->t('Entity type'),
        'bundle' => $this->t('Bundle'),
        'status' => $this->t('Status'),
        'operations' => $this->t('Operations'),
      ],
      '#rows' => $rows,
      '#empty' => $this->t('No fields available.'),
      '#cache' => [
        'tags' => [
          // Invalidate when fields change.
          'config:field_config_list',
          // Or when file entities change.
          'file_list',
        ],
      ],
    ];

    CacheableMetadata::createFromRenderArray($build)
      ->merge($cacheable_metadata)
      ->applyTo($build);

    return $build;
  }

  /**
   * Returns the label for a field config status.
   *
   * @param int $status
   *   The field config status.
   *
   * @return \Drupal\Component\Render\MarkupInterface
   *   A translatable label.
   */
  protected function getStatusLabel(int $status): MarkupInterface {
    return match ($status) {
      DefaultFilesFieldManagerInterface::FIELD_CONFIG_OK => $this->t('Ok'),
      DefaultFilesFieldManagerInterface::FIELD_CONFIG_MISSING_FILES => $this->t('Missing default files'),
      DefaultFilesFieldManagerInterface::FIELD_CONFIG_BROKEN => $this->t('Broken configuration'),
      DefaultFilesFieldManagerInterface::FIELD_CONFIG_FILE_VALIDATION_FAILED => $this->t('File validation errors'),
    };
  }

}
