<?php

declare(strict_types=1);

namespace Drupal\Tests\file_defaults_by_type_ui\Functional;

use Behat\Mink\Element\NodeElement;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\file_defaults_by_type\Traits\TestsFilesTrait;

/**
 * Test the form to add/replace default files.
 *
 * @group file_defaults_by_type
 */
class FieldDefaultFilesFormTest extends BrowserTestBase {

  use TestsFilesTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'entity_test',
    'file_defaults_by_type_test',
    'file_defaults_by_type_ui',
    'node',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests the form.
   */
  public function testForm(): void {
    // Generate files for the first 4 occurrences.
    $this->generateTestFiles(4);
    $this->assertFilesCount(4);

    $admin_user = $this->createUser(['administer default fields files']);
    $this->drupalLogin($admin_user);
    // Test the form for a field that has correct configuration.
    $this->drupalGet('/admin/config/media/file_defaults_by_type/field/node.article.node_article_field');

    $assert_session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // One upload field should be rendered.
    $this->assertCount(1, $page->findAll('css', '.form-managed-file'));
    $field_wrapper = $assert_session->elementExists('css', '.form-item-types-0-file');
    $field = $assert_session->fieldExists('File for Text', $field_wrapper);
    $this->assertDescriptionContainsString('Allowed types: txt.', $field_wrapper);
    $this->assertCurrentFile('example-2.txt', 'public://example-2.txt', $field_wrapper);

    // Load the file for later comparison.
    /** @var \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository */
    $entity_repository = \Drupal::service('entity.repository');
    /** @var \Drupal\file\FileInterface $original_file */
    $original_file = $entity_repository->loadEntityByUuid('file', 'uuid-2');
    $this->assertEquals(8, $original_file->getSize());
    $this->assertEquals('public://example-2.txt', $original_file->getFileUri());

    // Since we imported the configuration and created the file programmatically
    // without adding file usage, the current usage count for the file should be
    // zero.
    $file_usage = \Drupal::service('file.usage');
    $this->assertEquals([], $file_usage->listUsage($original_file));

    // Submitting the form without any changes will not replace the file.
    $page->pressButton('Submit');
    $this->assertCurrentFile('example-2.txt', 'public://example-2.txt', $field_wrapper);

    // Replace the file with a new one.
    file_put_contents('temporary://new-file.txt', $this->randomMachineName(12));
    $field->attachFile('temporary://new-file.txt');
    $assert_session->buttonExists('Upload', $field_wrapper)->press();
    // Submitting the file doesn't replace it yet, until the page is submitted.
    $this->assertCurrentFile('example-2.txt', 'public://example-2.txt', $field_wrapper);
    // Submit the form and check that the current file has been updated.
    $page->pressButton('Submit');
    // The file has been placed in a "templates" folder, respecting the
    // file_directory parameter of the file instance configuration.
    $this->assertCurrentFile('new-file.txt', 'public://templates/new-file.txt', $field_wrapper);

    // No new files have been created.
    $this->assertFilesCount(4);

    // Check that the new file data was saved into the original file entity.
    /** @var \Drupal\file\FileInterface $file */
    $file = $entity_repository->loadEntityByUuid('file', 'uuid-2');
    $this->assertEquals($original_file->id(), $file->id());
    $this->assertTrue($file->isPermanent());
    $this->assertEquals('public://templates/new-file.txt', $file->getFileUri());
    $this->assertEquals(12, $file->getSize());
    // The file usage data is added if missing.
    $this->assertEquals([
      'file_defaults_by_type' => [
        'field_config' => [
          'node.article.node_article_field' => 1,
        ],
      ],
    ], $file_usage->listUsage($file));
    // Check that the file was replaced.
    $this->assertFileExists('public://templates/new-file.txt');
    $this->assertFileDoesNotExist('public://example-2.txt');

    // Test a field where two upload fields are present.
    $this->drupalGet('/admin/config/media/file_defaults_by_type/field/node.page.node_page_field');
    $this->assertCount(2, $page->findAll('css', '.form-managed-file'));
    $field_one_wrapper = $assert_session->elementExists('css', '.form-item-types-0-file');
    $assert_session->fieldExists('File for Text', $field_one_wrapper);
    $this->assertDescriptionContainsString('Allowed types: txt ods.', $field_one_wrapper);
    $this->assertCurrentFile('example-3.txt', 'public://example-3.txt', $field_one_wrapper);

    $field_two_wrapper = $assert_session->elementExists('css', '.form-item-types-1-file');
    $field_two = $assert_session->fieldExists('File for Spreadsheet', $field_two_wrapper);
    $this->assertDescriptionContainsString('Allowed types: txt ods.', $field_two_wrapper);
    $this->assertCurrentFile('example-4.txt', 'public://example-4.txt', $field_two_wrapper);

    // Test that replacing the second field file doesn't change the first one.
    $original_file = $entity_repository->loadEntityByUuid('file', 'uuid-4');
    $this->assertEquals(8, $original_file->getSize());
    $this->assertEquals('public://example-4.txt', $original_file->getFileUri());
    // Since the files were created programmatically, there is no file usage
    // in this test execution. We already tested in the previous scenario that
    // the file usage will be added when missing. Now we add the initial file
    // usage programmatically so we can verify that the usage counter will stay
    // at 1.
    $file_usage->add($original_file, 'file_defaults_by_type', 'field_config', 'node.page.node_page_field');

    file_put_contents('temporary://bigger-file.txt', $this->randomMachineName(24));
    $field_two->attachFile('temporary://bigger-file.txt');
    $assert_session->buttonExists('Upload', $field_two_wrapper)->press();
    $page->pressButton('Submit');
    $this->assertCurrentFile('example-3.txt', 'public://example-3.txt', $field_one_wrapper);
    $this->assertCurrentFile('bigger-file.txt', 'public://bigger-file.txt', $field_two_wrapper);

    // Check that the file with UUID uuid-4 has been updated.
    $this->assertFilesCount(4);
    $file = $entity_repository->loadEntityByUuid('file', 'uuid-4');
    $this->assertEquals($original_file->id(), $file->id());
    $this->assertTrue($file->isPermanent());
    $this->assertEquals('public://bigger-file.txt', $file->getFileUri());
    $this->assertEquals(24, $file->getSize());
    $this->assertEquals([
      'file_defaults_by_type' => [
        'field_config' => [
          'node.page.node_page_field' => 1,
        ],
      ],
    ], $file_usage->listUsage($file));
    // Check that the file was replaced.
    $this->assertFileExists('public://bigger-file.txt');
    $this->assertFileDoesNotExist('public://example-4.txt');

    // When a file is missing, that field is going to be marked as required.
    $file->delete();
    $this->drupalGet('/admin/config/media/file_defaults_by_type/field/node.page.node_page_field');
    $page->pressButton('Submit');
    $assert_session->statusMessageContains('File for Spreadsheet field is required.', 'error');
    $this->assertDescriptionContainsString('No default file currently attached.', $field_two_wrapper);
  }

  /**
   * Asserts the current file attached to a field.
   *
   * @param string $expected_filename
   *   The expected file name.
   * @param string $expected_uri
   *   The expected file URI.
   * @param \Behat\Mink\Element\NodeElement $field_wrapper
   *   The container where to look for the file information.
   */
  protected function assertCurrentFile(string $expected_filename, string $expected_uri, NodeElement $field_wrapper): void {
    $assert_session = $this->assertSession();
    $description = $assert_session->elementExists('css', '.description', $field_wrapper);
    $link = $assert_session->elementExists('named', ['link', $expected_filename], $description);
    $this->assertEquals(
      \Drupal::service('file_url_generator')->generate($expected_uri)->toString(),
      $link->getAttribute('href'),
    );
    $this->assertStringContainsString("Current file: {$expected_filename}", $description->getText());
  }

  /**
   * Asserts that a file field description contains a string.
   *
   * @param string $expected_text
   *   The expected string to search for.
   * @param \Behat\Mink\Element\NodeElement $field_wrapper
   *   The file field wrapper.
   */
  protected function assertDescriptionContainsString(string $expected_text, NodeElement $field_wrapper): void {
    $this->assertStringContainsString(
      $expected_text,
      $this->assertSession()->elementExists('css', '.description', $field_wrapper)->getText(),
    );
  }

}
