<?php

declare(strict_types=1);

namespace Drupal\Tests\file_defaults_by_type_ui\Functional;

use Behat\Mink\Element\NodeElement;
use Drupal\Core\Url;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\file_defaults_by_type\Traits\TestsFilesTrait;

/**
 * Test to check fields list page.
 *
 * @group file_defaults_by_type
 */
class FieldsListPageTest extends BrowserTestBase {

  use TestsFilesTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'file_defaults_by_type_ui',
    'node',
    'entity_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Once enabled block module, we need to place the title.
    $this->drupalPlaceBlock('page_title_block');
    // Add tasks block to check task links.
    $this->drupalPlaceBlock('local_tasks_block');
  }

  /**
   * Tests the field list page.
   */
  public function testListPage(): void {
    $administrator = $this->drupalCreateUser([
      'access administration pages',
      'administer site configuration',
      'administer default fields files',
    ]);
    $this->drupalLogin($administrator);

    // Assert page message when there is no fields.
    $this->drupalGet('admin/config/media/file_defaults_by_type');
    $assert_session = $this->assertSession();
    $assert_session->pageTextContains('No fields available.');

    // Install the field definitions and bundles.
    \Drupal::service('module_installer')->install(['file_defaults_by_type_test']);
    // Generate test files.
    $this->generateTestFiles(4);
    // Skip the file number 5 to trigger a file missing report.
    $this->generateTestFile(6);

    // Check that the page contains the blocks for each bundle.
    $this->drupalGet('admin/config/media/file_defaults_by_type');
    $table = $assert_session->elementExists('css', 'main table');
    $rows = $table->findAll('xpath', '//tbody/tr');
    $this->assertCount(6, $rows);
    $this->assertListTableRow($rows[0], [
      'Label event field',
      'entity_test_event_field',
      'entity_test',
      'event',
      'Missing default files',
    ], Url::fromUserInput('/admin/config/media/file_defaults_by_type/field/entity_test.event.entity_test_event_field'));
    $this->assertListTableRow($rows[1], [
      'Label field for markdown',
      'field_file_markdown',
      'entity_test',
      'event',
      'File validation errors',
    ], Url::fromUserInput('/admin/config/media/file_defaults_by_type/field/entity_test.event.field_file_markdown'));
    $this->assertListTableRow($rows[2], [
      'Label article field',
      'node_article_field',
      'node',
      'article',
      'Ok',
    ], Url::fromUserInput('/admin/config/media/file_defaults_by_type/field/node.article.node_article_field'));
    $this->assertListTableRow($rows[3], [
      'field_with_empty',
      'field_with_empty',
      'node',
      'page',
      'Broken configuration',
    ]);
    $this->assertListTableRow($rows[4], [
      'field_with_invalid',
      'field_with_invalid',
      'node',
      'page',
      'Broken configuration',
    ]);
    $this->assertListTableRow($rows[5], [
      'Label page field',
      'node_page_field',
      'node',
      'page',
      'Ok',
    ], Url::fromUserInput('/admin/config/media/file_defaults_by_type/field/node.page.node_page_field'));

    $this->generateTestFile(5);
    $this->drupalGet('admin/config/media/file_defaults_by_type');
    $rows = $table->findAll('xpath', '//tbody/tr');
    $this->assertCount(6, $rows);
    // The first row now reports an OK status.
    $this->assertListTableRow($rows[0], [
      'Label event field',
      'entity_test_event_field',
      'entity_test',
      'event',
      'Ok',
    ], Url::fromUserInput('/admin/config/media/file_defaults_by_type/field/entity_test.event.entity_test_event_field'));
  }

  /**
   * Asserts a single row in the admin list table.
   *
   * @param \Behat\Mink\Element\NodeElement $row
   *   The row element.
   * @param array $expected
   *   A list of expected text content for each cell, up to the 5th one.
   * @param \Drupal\Core\Url|null $url
   *   An Url object if the 6th cell should contain one.
   */
  protected function assertListTableRow(NodeElement $row, array $expected, ?Url $url = NULL): void {
    // We expect 5 values to be passed in the array.
    $this->assertCount(5, $expected);
    $cells = $row->findAll('css', 'td');
    foreach ($expected as $i => $value) {
      $this->assertEquals($value, $cells[$i]->getText());
    }

    if ($url) {
      $this->assertEquals($url->toString(), $this->assertSession()->elementExists('css', 'a', $cells[5])->getAttribute('href'));
    }
    else {
      $this->assertSession()->elementNotExists('css', 'a', $cells[5]);
    }
  }

}
