<?php

declare(strict_types=1);

namespace Drupal\Tests\file_defaults_by_type_ui\Kernel;

use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AnonymousUserSession;

/**
 * Tests the access to routes provided by the module.
 *
 * @group file_defaults_by_type
 */
class RouteAccessTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'file_defaults_by_type_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig(['file_defaults_by_type_test']);

    // Create a first user to serve as UID 1.
    $this->createUser();
  }

  /**
   * Tests that the permission provided by the module is applied to routes.
   */
  public function testRouteAccess(): void {
    // Prepare the parameters for all the routes:
    // - field list;
    // - a field with a valid config;
    // - a field with valid config but missing files;
    // - a field with a broken config.
    // - a field with disabled types but leftover config.
    $admin_list_route_params = [
      'file_defaults_by_type_ui.admin.field_list',
      [],
    ];
    $valid_config_field_route_params = [
      'file_defaults_by_type_ui.admin.field_default_files',
      [
        'field' => 'node.page.node_page_field',
      ],
    ];
    $missing_file_config_route_params = [
      'file_defaults_by_type_ui.admin.field_default_files',
      [
        'field' => 'entity_test.event.entity_test_event_field',
      ],
    ];
    $broken_config_route_params = [
      'file_defaults_by_type_ui.admin.field_default_files',
      [
        'field' => 'node.page.field_with_empty',
      ],
    ];
    $disabled_config_route_params = [
      'file_defaults_by_type_ui.admin.field_default_files',
      [
        'field' => 'node.page.field_with_disabled',
      ],
    ];

    // Unprivileged users cannot access any routes offered by this module.
    $anonymous = new AnonymousUserSession();
    $this->assertFalse($this->checkRouteAccess($anonymous, ...$admin_list_route_params));
    $this->assertFalse($this->checkRouteAccess($anonymous, ...$valid_config_field_route_params));
    $this->assertFalse($this->checkRouteAccess($anonymous, ...$missing_file_config_route_params));
    $this->assertFalse($this->checkRouteAccess($anonymous, ...$broken_config_route_params));
    $this->assertFalse($this->checkRouteAccess($anonymous, ...$disabled_config_route_params));

    // Registered users without permissions cannot access the routes.
    $authenticated_user = $this->createUser();
    $this->assertFalse($this->checkRouteAccess($authenticated_user, ...$admin_list_route_params));
    $this->assertFalse($this->checkRouteAccess($authenticated_user, ...$valid_config_field_route_params));
    $this->assertFalse($this->checkRouteAccess($authenticated_user, ...$missing_file_config_route_params));
    $this->assertFalse($this->checkRouteAccess($authenticated_user, ...$broken_config_route_params));
    $this->assertFalse($this->checkRouteAccess($authenticated_user, ...$disabled_config_route_params));

    $admin_user = $this->createUser(['administer default fields files']);
    $this->assertTrue($this->checkRouteAccess($admin_user, ...$admin_list_route_params));
    $this->assertTrue($this->checkRouteAccess($admin_user, ...$valid_config_field_route_params));
    $this->assertTrue($this->checkRouteAccess($admin_user, ...$missing_file_config_route_params));
    // Access to invalid and disabled fields is still prevented.
    $this->assertFalse($this->checkRouteAccess($admin_user, ...$broken_config_route_params));
    $this->assertFalse($this->checkRouteAccess($admin_user, ...$disabled_config_route_params));
  }

  /**
   * Returns access to the route with a set of parameters.
   *
   * @param \Drupal\Core\Session\AccountInterface $user
   *   A user account to use for the check. Null to use anonymous.
   * @param string $route_name
   *   The route name.
   * @param array $route_parameters
   *   The route parameters.
   *
   * @return bool
   *   True if access is allowed, false otherwise.
   */
  protected function checkRouteAccess(AccountInterface $user, string $route_name, array $route_parameters): bool {
    $this->setCurrentUser($user ?? new AnonymousUserSession());

    $access_check = $this->container->get('access_manager')->checkNamedRoute(
      $route_name,
      $route_parameters,
      $user,
      TRUE,
    );

    return $access_check->isAllowed();
  }

}
