<?php

declare(strict_types=1);

namespace Drupal\file_defaults_by_type;

use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\field\FieldConfigInterface;
use Drupal\file\FileInterface;
use Drupal\file\Validation\FileValidatorInterface;

/**
 * The default file field manager.
 */
class DefaultFilesFieldManager implements DefaultFilesFieldManagerInterface {

  use FileValidatorsTrait;

  /**
   * Constructs a new instance of this class.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity manager.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The entity repository.
   * @param \Drupal\file\Validation\FileValidatorInterface $fileValidator
   *   The file validator.
   */
  public function __construct(
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly EntityRepositoryInterface $entityRepository,
    protected readonly FileValidatorInterface $fileValidator,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getEnabledFields(): array {
    // The ::loadByProperties() method of the field config storage doesn't use
    // a traditional entity query for retrieving the fields, and it has limited
    // support for nested value retrieval.
    // Since we don't need deleted fields in this interface, we run the query
    // manually.
    $storage = $this->entityTypeManager->getStorage('field_config');
    $query = $storage->getQuery()
      ->accessCheck(FALSE)
      ->condition('field_type', 'file')
      ->condition('third_party_settings.file_defaults_by_type.enabled', TRUE);
    $result = $query->execute();

    return $result ? $storage->loadMultiple($result) : [];
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldConfigurationStatus(FieldConfigInterface $field): int {
    $third_party_settings = $field->getThirdPartySettings('file_defaults_by_type');

    if (!$third_party_settings['enabled']) {
      throw new \InvalidArgumentException('This method should be invoked only on fields where the default file types are enabled.');
    }

    // At least a type should be declared.
    if (empty($third_party_settings['types'])) {
      return self::FIELD_CONFIG_BROKEN;
    }

    // If any type is marked as not valid, the configuration is invalid.
    $valid_types = FileTypesHelper::getValidTypes($third_party_settings);
    if (count($third_party_settings['types']) !== count($valid_types)) {
      return self::FIELD_CONFIG_BROKEN;
    }

    $validators = $this->getUploadValidators($field->getSettings());
    foreach ($valid_types as $type) {
      $file = $this->entityRepository->loadEntityByUuid('file', $type['default']);

      if (!$file instanceof FileInterface) {
        return self::FIELD_CONFIG_MISSING_FILES;
      }

      if (count($this->fileValidator->validate($file, $validators))) {
        return self::FIELD_CONFIG_FILE_VALIDATION_FAILED;
      }
    }

    return self::FIELD_CONFIG_OK;
  }

}
