<?php

declare(strict_types=1);

namespace Drupal\file_defaults_by_type;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\file\Element\ManagedFile;
use Drupal\file\FileUsage\FileUsageInterface;
use Drupal\file\Validation\FileValidatorInterface;
use Drupal\multivalue_form_element\Element\MultiValue;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class to manage file field settings.
 *
 * @internal
 */
final class SettingsFormAlter implements ContainerInjectionInterface {

  use DependencySerializationTrait;
  use FileLocationTrait;
  use FileValidatorsTrait;
  use StringTranslationTrait;

  /**
   * Creates a new instance of this class.
   *
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The entity type manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Drupal\file\Validation\FileValidatorInterface $fileValidator
   *   The file validator.
   * @param \Drupal\file\FileUsage\FileUsageInterface $fileUsage
   *   The file usage.
   */
  public function __construct(
    protected readonly EntityRepositoryInterface $entityRepository,
    protected readonly RendererInterface $renderer,
    protected readonly FileValidatorInterface $fileValidator,
    protected readonly FileUsageInterface $fileUsage,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('entity.repository'),
      $container->get('renderer'),
      $container->get('file.validator'),
      $container->get('file.usage')
    );
  }

  /**
   * Alters the form to add types settings.
   *
   * @param array $form
   *   The form element to process.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function settingsFormAlter(array &$form, FormStateInterface $form_state): void {
    $storage = $form_state->getStorage();

    if ($storage === NULL || !isset($storage['field_config']) || $storage['field_config']->get('field_type') !== 'file') {
      return;
    }

    /** @var \Drupal\field\Entity\FieldConfig $field_config */
    $field_config = $storage['field_config'];
    $default_settings = $field_config->getThirdPartySettings('file_defaults_by_type');

    $settings['file_defaults_by_type']['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Default file types enabled'),
      '#default_value' => $enabled = $default_settings['enabled'] ?? FALSE,
      '#description' => $this->t('Enable file types: label and default file.'),
    ];

    // Prepare default values.
    $default_types = $enabled ? $default_settings['types'] : [];
    foreach ($default_types as &$default_type) {
      // Replace UUID with fids in default file.
      // Values are converted into UUIDs later on validation.
      if (!empty($default_type['default'])) {
        $file = $this->entityRepository->loadEntityByUuid('file', $default_type['default']);
        $default_type['default'] = $file ? [$file->id()] : [];
      }
    }

    // @todo Label should be required in all types.
    $settings['file_defaults_by_type']['types'] = [
      '#type' => 'multivalue',
      '#title' => $this->t('Types'),
      '#cardinality' => MultiValue::CARDINALITY_UNLIMITED,
      '#default_value' => $default_types,
      '#states' => [
        'visible' => [
          ':input[name="third_party_settings[file_defaults_by_type][enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $settings['file_defaults_by_type']['types']['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#description' => $this->t("Label displayed to the user to select the type used."),
    ];

    // Get upload validators based on field settings.
    $field_settings = $field_config->getSettings();
    // @todo Use Drupal\file\Validation\FileValidatorSettingsTrait when support
    // for Drupal Core 10.2.x is dropped.
    $upload_validators = $this->getUploadValidators($field_settings);

    $settings['file_defaults_by_type']['types']['default'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Default file'),
      '#field_name' => $field_config->getName(),
      '#upload_location' => $this->getUploadLocation($field_settings),
      '#element_validate' => [
        [ManagedFile::class, 'validateManagedFile'],
        [static::class, 'validateDefaultFileForm'],
      ],
      '#upload_validators' => $upload_validators,
      '#description' => [
        '#theme' => 'file_upload_help',
        '#description' => 'File to be added if no file is uploaded.',
        '#upload_validators' => $upload_validators,
        '#cardinality' => 1,
      ],
    ];

    $form['#validate'][] = [$this, 'validateForm'];

    $form['actions']['submit']['#submit'][] = [$this, 'processFiles'];

    // Add settings to the form.
    $form['third_party_settings'] += $settings;
  }

  /**
   * Validates the form.
   *
   * @param array $form
   *   The form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $settings = $form_state->getValue('third_party_settings');
    if (empty($settings['file_defaults_by_type']['enabled'])) {
      // Clean up possible leftover configuration after disabling the
      // functionality.
      $form_state->setValueForElement($form['third_party_settings']['file_defaults_by_type']['types'], []);
      return;
    }

    $types = $settings['file_defaults_by_type']['types'] ?? [];
    if (empty($types)) {
      $form_state->setError(
        $form['third_party_settings']['file_defaults_by_type'],
        $this->t('At least one type is required when the functionality is enabled.'),
      );
      return;
    }

    /** @var \Drupal\field\FieldConfigInterface $config_entity */
    $config_entity = $form_state->getFormObject()->getEntity();
    $validators = $this->getUploadValidators($config_entity->getSettings());

    foreach ($types as $i => $type) {
      $is_empty_label = ($type['label'] ?? '') === '';
      $is_empty_file = ($type['default'] ?? '') === '';

      if ($is_empty_label xor $is_empty_file) {
        $field = $is_empty_label ? 'label' : 'default';
        $form_state->setError(
          $form['third_party_settings']['file_defaults_by_type']['types'][$i][$field],
          new TranslatableMarkup('Both <em>label</em> and <em>default file</em> fields are required when either of them is filled in.'),
        );
      }

      // Re-run validations for already uploaded files.
      if (!$is_empty_file && ($file = $this->entityRepository->loadEntityByUuid('file', $type['default']))) {
        /** @var \Drupal\file\FileInterface $file */
        if ($violations = $this->fileValidator->validate($file, $validators)) {
          /** @var \Symfony\Component\Validator\ConstraintViolationListInterface $violations */
          foreach ($violations as $violation) {
            $form_state->setError(
              $form['third_party_settings']['file_defaults_by_type']['types'][$i]['default'],
              $violation->getMessage(),
            );
          }
        }
      }
    }
  }

  /**
   * Submit handler to process uploaded files for default types.
   *
   * Makes files permanent and updates files usage.
   *
   * @param array $form
   *   The form element to process.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function processFiles(array &$form, FormStateInterface $form_state): void {
    /** @var \Drupal\field\Entity\FieldConfig $field_config */
    $field_config = $form_state->getStorage()['field_config'];
    $settings = $form_state->getValue('third_party_settings');

    $types = $settings['file_defaults_by_type']['types'];

    // Delete file usage for removed files.
    $original_settings = $field_config->getThirdPartySettings('file_defaults_by_type');
    $original_uuids = array_column($original_settings['types'], 'default');
    $removed_uuids = array_diff($original_uuids, array_column($types, 'default'));
    foreach ($removed_uuids as $uuid) {
      if ($file = $this->entityRepository->loadEntityByUuid('file', $uuid)) {
        /** @var \Drupal\file\FileInterface $file */
        $this->fileUsage->delete($file, 'file_defaults_by_type', 'field_config', $field_config->id());
      }
    }

    if (!FileTypesHelper::isEnabled($settings['file_defaults_by_type'])) {
      return;
    }

    foreach ($types as $type) {
      if (empty($type['default'])) {
        continue;
      }

      /** @var \Drupal\file\Entity\File $file */
      $file = $this->entityRepository->loadEntityByUuid('file', $type['default']);

      // File doesn't exist or is already a permanent file.
      if ($file === NULL || $file->isPermanent()) {
        continue;
      }

      // Make the submitted file permanent.
      $file->setPermanent();
      $file->save();
      $this->fileUsage->add($file, 'file_defaults_by_type', 'field_config', $field_config->id());
    }
  }

  /**
   * Validates the managed_file element for the default File form.
   *
   * Converts fid into UUID.
   *
   * @param array $element
   *   The form element to process.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public static function validateDefaultFileForm(array &$element, FormStateInterface $form_state): void {
    $value = '';

    if (isset($element['fids']['#value'][0])) {
      /** @var \Drupal\file\Entity\File $file */
      if ($file = \Drupal::entityTypeManager()->getStorage('file')->load($element['fids']['#value'][0])) {
        $value = $file->uuid();
      }
    }

    $form_state->setValueForElement($element, $value);
  }

}
