<?php

declare(strict_types=1);

namespace Drupal\Tests\file_defaults_by_type\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\TestFileCreationTrait;
use Drupal\Tests\file_defaults_by_type\Traits\DefaultFilesFieldTrait;
use Drupal\field\Entity\FieldConfig;
use Drupal\file\Entity\File;

/**
 * Tests the file types settings.
 *
 * @group file_defaults_by_type
 */
class FieldSettingsFormTest extends WebDriverTestBase {

  use DefaultFilesFieldTrait;
  use TestFileCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'file_defaults_by_type',
    'field_ui',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Test settings added for File field form.
   */
  public function testSettingsForm(): void {
    $this->drupalCreateContentType(['type' => 'article', 'name' => 'Article']);
    $this->createDefaultFilesField(
      'file_field',
      'node',
      'article',
      [
        'cardinality' => 1,
        'type' => 'file',
      ],
      [
        'label' => 'File field',
        'settings' => [
          'file_extensions' => 'txt',
        ],
      ],
    );

    $adminUser = $this->drupalCreateUser([
      'access administration pages',
      'administer node fields',
    ]);
    $this->drupalLogin($adminUser);
    $assert_session = $this->assertSession();
    $page = $this->getSession()->getPage();
    $test_files = $this->getTestFiles('text');

    // Add field using 'File upload' type to check changes on field form.
    $this->drupalGet('/admin/structure/types/manage/article/fields/node.article.file_field');

    // Fields are present.
    $enabled_field = $assert_session->fieldExists('Default file types enabled');
    $types_selector = '[data-drupal-selector="edit-third-party-settings-file-defaults-by-type-types"]';
    $assert_session->elementTextNotContains('css', $types_selector, 'Types');

    // After checking enabled field: types are displayed.
    $enabled_field->check();
    $assert_session->elementTextContains('css', $types_selector, 'Types');

    // When the functionality is enabled, at least one type must be provided.
    $page->pressButton('Save settings');
    $assert_session->statusMessageContains('At least one type is required when the functionality is enabled.', 'error');

    // Names helper functions.
    $field_name = fn (string $field, string $delta): string => "third_party_settings[file_defaults_by_type][types][$delta][$field]";
    $upload_name = fn (string $delta): string => "third_party_settings_file_defaults_by_type_types_{$delta}_default";

    // When a label is entered, the default file field is required to be filled.
    $types = $assert_session->elementExists('css', $types_selector);
    $assert_session->fieldExists($field_name('label', '0'), $types)->setValue('Text file');
    $page->pressButton('Save settings');
    $assert_session->statusMessageContains('Both label and default file fields are required when either of them is filled in.', 'error');

    // Only a file with one of the allowed file extensions can be uploaded.
    file_put_contents('public://markdown.md', $this->randomMachineName(12));
    $page->attachFileToField("files[{$upload_name('0')}]", \Drupal::service('file_system')->realpath('public://markdown.md'));
    $this->assertTrue($assert_session->waitForText('The selected file markdown.md cannot be uploaded. Only files with the following extensions are allowed: txt.'));

    // Attach a valid file.
    $page->attachFileToField("files[{$upload_name('0')}]", \Drupal::service('file_system')->realpath($test_files[0]->uri));
    $assert_session->assertWaitOnAjaxRequest();
    $this->assertNotNull($assert_session->waitForElementVisible('css', "[name='{$upload_name('0')}_remove_button']"));

    // A file entity has been created, but it's not permanent.
    $file_1_id = $assert_session->elementExists('css', "[name='{$field_name('default', '0')}[fids]']")->getValue();
    $file_1 = File::load($file_1_id);
    $this->assertFalse($file_1->isPermanent());
    // File usage is not created until the page is saved.
    /** @var \Drupal\file\FileUsage\FileUsageInterface $file_usage */
    $file_usage = \Drupal::service('file.usage');
    $this->assertEquals([], $file_usage->listUsage($file_1));

    // Save the page and check that the file entity is now permanent.
    $page->pressButton('Save settings');
    $assert_session->statusMessageContains('Saved File field configuration.', 'status');

    \Drupal::entityTypeManager()->getStorage('file')->resetCache();
    $file_1 = File::load($file_1_id);
    $this->assertTrue($file_1->isPermanent());
    $this->assertEquals([
      'file_defaults_by_type' => [
        'field_config' => [
          'node.article.file_field' => 1,
        ],
      ],
    ], $file_usage->listUsage($file_1));

    // Edit again the file and check that the configuration has been persisted.
    $this->drupalGet('/admin/structure/types/manage/article/fields/node.article.file_field');
    $assert_session->checkboxChecked('Default file types enabled');
    $this->assertEquals('Text file', $assert_session->fieldExists($field_name('label', '0'), $types)->getValue());

    // Test that the type label is required if a default file is attached.
    $assert_session->fieldExists($field_name('label', '0'), $types)->setValue('');
    $page->pressButton('Save settings');
    $assert_session->statusMessageContains('Both label and default file fields are required when either of them is filled in.', 'error');

    // Reload the form to reset the changes.
    $this->drupalGet('/admin/structure/types/manage/article/fields/node.article.file_field');
    // Add a new file extension to the allowed list.
    $assert_session->fieldExists('Allowed file extensions')->setValue('txt, md');
    $page->pressButton('Save settings');
    $assert_session->statusMessageContains('Saved File field configuration.', 'status');

    $this->drupalGet('/admin/structure/types/manage/article/fields/node.article.file_field');
    $page->attachFileToField("files[{$upload_name('1')}]", \Drupal::service('file_system')->realpath('public://markdown.md'));
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->fieldExists($field_name('label', '1'), $types)->setValue('Markdown');
    // Store the file ID for later.
    $file_2_id = $assert_session->elementExists('css', "[name='{$field_name('default', '1')}[fids]']")->getValue();

    // Add another file type.
    $assert_session->buttonExists('Add another item')->press();
    $assert_session->assertWaitOnAjaxRequest();
    $page->attachFileToField("files[{$upload_name('2')}]", \Drupal::service('file_system')->realpath($test_files[1]->uri));
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->fieldExists($field_name('label', '2'), $types)->setValue('Another type');
    $file_3_id = $assert_session->elementExists('css', "[name='{$field_name('default', '2')}[fids]']")->getValue();
    $page->pressButton('Save settings');
    $assert_session->statusMessageContains('Saved File field configuration.', 'status');
    $file_2 = File::load($file_2_id);
    $this->assertTrue($file_2->isPermanent());
    $this->assertEquals([
      'file_defaults_by_type' => [
        'field_config' => [
          'node.article.file_field' => 1,
        ],
      ],
    ], $file_usage->listUsage($file_2));
    $file_3 = File::load($file_3_id);
    $this->assertTrue($file_3->isPermanent());
    $this->assertEquals([
      'file_defaults_by_type' => [
        'field_config' => [
          'node.article.file_field' => 1,
        ],
      ],
    ], $file_usage->listUsage($file_3));
    // File usage for file 1 stayed the same.
    $this->assertEquals([
      'file_defaults_by_type' => [
        'field_config' => [
          'node.article.file_field' => 1,
        ],
      ],
    ], $file_usage->listUsage($file_1));

    // Remove one of the extensions from the allowed list.
    $this->drupalGet('/admin/structure/types/manage/article/fields/node.article.file_field');
    $assert_session->fieldExists('Allowed file extensions')->setValue('txt');
    $page->pressButton('Save settings');
    // Files that have been already uploaded and do not match anymore the
    // allowed extension list will trigger an error.
    $assert_session->statusMessageContains('Only files with the following extensions are allowed: txt.', 'error');

    // All file upload validations are executed.
    $this->drupalGet('/admin/structure/types/manage/article/fields/node.article.file_field');
    $assert_session->fieldExists('Maximum upload size')->setValue('24');
    $page->pressButton('Save settings');
    $assert_session->statusMessageContains('The file is 1 KB exceeding the maximum file size of 24 bytes.', 'error');
    $assert_session->statusMessageContains('The file is 16 KB exceeding the maximum file size of 24 bytes.', 'error');

    // Load the file configuration and check that the correct data is exported.
    $field_config = FieldConfig::load('node.article.file_field');
    $this->assertEquals([
      'enabled' => TRUE,
      'types' => [
        [
          'label' => 'Text file',
          'default' => $file_1->uuid(),
        ],
        [
          'label' => 'Markdown',
          'default' => $file_2->uuid(),
        ],
        [
          'label' => 'Another type',
          'default' => $file_3->uuid(),
        ],
      ],
    ], $field_config->getThirdPartySettings('file_defaults_by_type'));

    // Remove a type.
    $this->drupalGet('/admin/structure/types/manage/article/fields/node.article.file_field');
    $assert_session->fieldExists($field_name('label', '1'), $types)->setValue('');
    $assert_session->elementExists('css', "[name='{$upload_name('1')}_remove_button']")->press();
    $assert_session->assertWaitOnAjaxRequest();
    $page->pressButton('Save settings');
    $assert_session->statusMessageContains('Saved File field configuration.', 'status');
    // The usage for file 2 has been decremented, while the other were kept.
    $this->assertEquals([], $file_usage->listUsage($file_2));
    $this->assertEquals([
      'file_defaults_by_type' => [
        'field_config' => [
          'node.article.file_field' => 1,
        ],
      ],
    ], $file_usage->listUsage($file_1));
    $this->assertEquals([
      'file_defaults_by_type' => [
        'field_config' => [
          'node.article.file_field' => 1,
        ],
      ],
    ], $file_usage->listUsage($file_3));

    $field_config_storage = \Drupal::entityTypeManager()->getStorage('field_config');
    $field_config_storage->resetCache();
    $field_config = $field_config_storage->load('node.article.file_field');
    $this->assertEquals([
      'enabled' => TRUE,
      'types' => [
        [
          'label' => 'Text file',
          'default' => $file_1->uuid(),
        ],
        [
          'label' => 'Another type',
          'default' => $file_3->uuid(),
        ],
      ],
    ], $field_config->getThirdPartySettings('file_defaults_by_type'));

    // Disable the types and check that the configuration is cleaned.
    $this->drupalGet('/admin/structure/types/manage/article/fields/node.article.file_field');
    $assert_session->fieldExists('Default file types enabled')->uncheck();
    $page->pressButton('Save settings');
    $assert_session->statusMessageContains('Saved File field configuration.', 'status');

    $field_config_storage->resetCache();
    $field_config = $field_config_storage->load('node.article.file_field');
    $this->assertEquals([
      'enabled' => FALSE,
      'types' => [],
    ], $field_config->getThirdPartySettings('file_defaults_by_type'));
  }

}
