<?php

declare(strict_types=1);

namespace Drupal\Tests\file_defaults_by_type\FunctionalJavascript;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\file\Entity\File;

/**
 * Tests the file field widget, single and multi-valued, using AJAX upload.
 *
 * @group file_defaults_by_type
 */
class WidgetUploadTest extends FieldTestBase {

  /**
   * Tests default_file_choice for single valued widget.
   */
  public function testSingleValuedWidget(): void {
    $assert_session = $this->assertSession();
    $page = $this->getSession()->getPage();

    $this->drupalGet('node/add/article');
    $title = $this->randomString();
    $page->findField('title[0][value]')->setValue($title);
    // Check that existing files are only generated.
    $this->assertFilesCount(3);
    $uid = (int) \Drupal::currentUser()->id();

    // Check that value is set using attach.
    $assert_session->selectExists('File type')->selectOption('File 2');
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->buttonExists('Attach')->press();
    $assert_session->waitForElementVisible('css', '[name="test_file_field_0_remove_button"]');
    $assert_session->hiddenFieldValueEquals('test_file_field[0][fids]', '4');
    $this->assertFilesCount(4);
    $this->assertCopyFile(2, 4, $uid);
    $page->pressButton('Save');
    $assert_session->statusMessageContains("Article $title has been created.", 'status');
    $assert_session->linkExists('example-2.txt');

    // Check that file usage is tracked normally.
    /** @var \Drupal\file\FileUsage\FileUsageInterface $file_usage */
    $file_usage = \Drupal::service('file.usage');
    $this->assertEquals([
      'file' => [
        'node' => [
          // The key "1" is the node ID of the page we just created.
          1 => 1,
        ],
      ],
    ], $file_usage->listUsage(File::load(4)));

    // Check that the value can be replaced.
    $this->drupalGet('node/1/edit');
    $assert_session->elementExists('css', '[name="test_file_field_0_remove_button"]')->press();
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->selectExists('File type')->selectOption('File 3');
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->buttonExists('Attach')->press();
    $assert_session->waitForElementVisible('css', '[name="test_file_field_0_remove_button"]');
    $assert_session->hiddenFieldValueEquals('test_file_field[0][fids]', '5');
    $this->assertFilesCount(5);
    $this->assertCopyFile(3, 5, $uid);
    $page->pressButton('Save');
    $assert_session->statusMessageContains("Article $title has been updated.", 'status');
    $assert_session->linkExists('example-3.txt');
    $assert_session->linkNotExists('example-2.txt');

    // Check that we can change the type, but we can opt to leave it empty.
    $this->drupalGet('node/1/edit');
    $assert_session->elementExists('css', '[name="test_file_field_0_remove_button"]')->press();
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->selectExists('File type')->selectOption('File 2');
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->selectExists('File type')->selectOption('File 1');
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->selectExists('File type')->selectOption('File 3');
    $assert_session->assertWaitOnAjaxRequest();
    $this->assertFilesCount(5);
    $page->pressButton('Save');
    $assert_session->statusMessageContains("Article $title has been updated.", 'status');
    $assert_session->linkNotExists('example-1.txt');
    $assert_session->linkNotExists('example-2.txt');
    $assert_session->linkNotExists('example-3.txt');

    // Check required field, try with an empty value, set value and save.
    FieldConfig::load('node.article.test_file_field')->setRequired(TRUE)->save();
    $this->drupalGet('node/1/edit');
    $assert_session->selectExists('File type')->selectOption('File 2');
    $assert_session->assertWaitOnAjaxRequest();
    $page->pressButton('Save');
    $assert_session->statusMessageContains('test_file_field field is required.', 'error');
    $assert_session->buttonExists('Attach')->press();
    $assert_session->waitForElementVisible('css', '[name="test_file_field_0_remove_button"]');
    $assert_session->hiddenFieldValueEquals('test_file_field[0][fids]', '6');
    $this->assertFilesCount(6);
    $this->assertCopyFile(2, 6, $uid);
    $page->pressButton('Save');
    $assert_session->statusMessageContains("Article $title has been updated.", 'status');
    $assert_session->linkExists('example-2.txt');
  }

  /**
   * Tests default_file_choice for multivalued valued widget.
   */
  public function testMultiValuedWidget(): void {
    FieldStorageConfig::load('node.test_file_field')->setCardinality(-1)->save();
    $assert_session = $this->assertSession();
    $page = $this->getSession()->getPage();

    $this->drupalGet('node/add/article');
    $title = $this->randomString();
    $page->findField('title[0][value]')->setValue($title);
    $this->assertFilesCount(3);
    $uid = (int) \Drupal::currentUser()->id();

    // Check that multiple values are set using attach.
    $assert_session->buttonExists('Attach')->press();
    $assert_session->waitForElementVisible('css', '[name="test_file_field_0_remove_button"]');
    $assert_session->selectExists('File type')->selectOption('File 2');
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->buttonExists('Attach')->press();
    $assert_session->waitForElementVisible('css', '[name="test_file_field_1_remove_button"]');
    $assert_session->selectExists('File type')->selectOption('File 3');
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->buttonExists('Attach')->press();
    $assert_session->waitForElementVisible('css', '[name="test_file_field_2_remove_button"]');
    $assert_session->hiddenFieldValueEquals('test_file_field[0][fids]', '4');
    $assert_session->hiddenFieldValueEquals('test_file_field[1][fids]', '5');
    $assert_session->hiddenFieldValueEquals('test_file_field[2][fids]', '6');
    $this->assertFilesCount(6);
    $this->assertCopyFile(1, 4, $uid);
    $this->assertCopyFile(2, 5, $uid);
    $this->assertCopyFile(3, 6, $uid);
    $page->pressButton('Save');
    $assert_session->statusMessageContains("Article $title has been created.", 'status');
    $assert_session->linkExists('example-1.txt');
    $assert_session->linkExists('example-2.txt');
    $assert_session->linkExists('example-3.txt');

    // Check the removal or all files, and empty value for the field.
    $this->drupalGet('node/1/edit');
    $assert_session->elementExists('css', '[name="test_file_field_0_remove_button"]')->press();
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->elementExists('css', '[name="test_file_field_0_remove_button"]')->press();
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->elementExists('css', '[name="test_file_field_0_remove_button"]')->press();
    $assert_session->assertWaitOnAjaxRequest();
    $this->assertFilesCount(6);
    $page->pressButton('Save');
    $assert_session->statusMessageContains("Article $title has been updated.", 'status');
    $assert_session->linkNotExists('example-1.txt');
    $assert_session->linkNotExists('example-2.txt');
    $assert_session->linkNotExists('example-3.txt');

    // Check required field, try with an empty value, set value and save.
    FieldConfig::load('node.article.test_file_field')->setRequired(TRUE)->save();
    $this->drupalGet('node/1/edit');
    $assert_session->selectExists('File type')->selectOption('File 2');
    $assert_session->assertWaitOnAjaxRequest();
    $page->pressButton('Save');
    $assert_session->statusMessageContains('test_file_field field is required.', 'error');
    $assert_session->buttonExists('Attach')->press();
    $assert_session->waitForElementVisible('css', '[name="test_file_field_0_remove_button"]');
    $assert_session->hiddenFieldValueEquals('test_file_field[0][fids]', '7');
    $this->assertCopyFile(2, 7, $uid);
    $this->assertFilesCount(7);
    $page->pressButton('Save');
    $assert_session->statusMessageContains("Article $title has been updated.", 'status');
    $assert_session->linkExists('example-2.txt');
    $assert_session->linkNotExists('example-1.txt');
    $assert_session->linkNotExists('example-3.txt');
  }

  /**
   * Disables the native browser validation for required fields.
   */
  protected function disableNativeBrowserRequiredFieldValidation(): void {
    $this->getSession()->executeScript('jQuery(":input[required]").prop("required", false);');
  }

  /**
   * Asserts the copy version of the file on widget upload.
   *
   * The values are the same except for the file URI, author and status.
   *
   * @param int $origin_fid
   *   The source File.
   * @param int $copy_fid
   *   The copy version of the File.
   * @param int $uid
   *   The user id from the author of the file.
   */
  protected function assertCopyFile(int $origin_fid, int $copy_fid, int $uid): void {
    $file_storage = \Drupal::entityTypeManager()->getStorage('file');

    $this->assertNotEquals($origin_fid, $copy_fid);

    /** @var \Drupal\file\Entity\File $origin */
    $origin = $file_storage->load($origin_fid);
    if ($origin === NULL) {
      throw new \InvalidArgumentException("The origin File with the fid:$origin_fid doesn't exist.");
    }

    /** @var \Drupal\file\Entity\File $copy */
    $copy = $file_storage->load($copy_fid);
    if ($copy === NULL) {
      throw new \InvalidArgumentException("The copied File with the fid:$copy_fid doesn't exist.");
    }

    // Check properties.
    $this->assertTrue($copy->isTemporary());
    $this->assertEquals($uid, $copy->getOwner()->id());
    $this->assertNotEquals($origin->getFileUri(), $copy->getFileUri());
    $this->assertEquals($origin->getFilename(), $copy->getFilename());
    $this->assertEquals($origin->getSize(), $copy->getSize());
    $this->assertEquals($origin->getMimeType(), $copy->getMimeType());
    $this->assertEquals(file_get_contents($origin->getFileUri()), file_get_contents($copy->getFileUri()));
  }

}
